/*
 * (C) Copyright 2003
 * Tait Electronics Limited, Christchurch, New Zealand
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>


#if defined(CONFIG_8xx)
#include <commproc.h>
#else
#error "Apparently a bad configuration, please fix."
#endif

#include <command.h>

extern short get_digio_value(void);

/* bit pattern sequence to be used in digital output test */
const static unsigned int pattern [] = {
    0x3f300,
    0x00000,
    0x00100,
    0x00200,
    0x01000,
    0x02000,
    0x04000,
    0x08000,
    0x10000,
    0x20000,
    0x3f200,
    0x3f100,
    0x3e300,
    0x3d300,
    0x3b300,
    0x37300,
    0x2f300,
    0x1f300};

#define PATTERNLEN	sizeof(pattern) / sizeof(pattern[0])



  /*
   * Test routines
   */


int do_digin ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	int l = 1;
	
	volatile immap_t *immr = (immap_t *) CFG_IMMR;


	immr->im_ioport.iop_pcso  &= ~PC_DIGIO_MASK;	/* assignment = general i/o */
	immr->im_ioport.iop_pcpar &= ~PC_DIGIO_MASK;	/* assignment = general i/o */
        immr->im_ioport.iop_pcdir &= ~PC_DIGIO_MASK;	/* direction = in */
        immr->im_ioport.iop_padir &= ~PA_1PPS_INPUT;	/* direction = in */


	/*
	 * Loop until control C pressed displaying input values
	 * Update display every .1 seconds or so
	 */
	printf("\nPress <control>C to exit\n");
	while (l) {
		if (ctrlc()) {
			l = 0;
		        break;
		}
		printf("Digital I/O value = %04X\r", get_digio_value());
		udelay (100000);				/* .1 sec sleep */
	}
	printf("\n");

	return 0;
}



int do_digout ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	int i;
	int l = 1;
	volatile immap_t *immr = (immap_t *) CFG_IMMR;

	immr->im_cpm.cp_pbodr &= ~PB_DIGIO_MASK;	/* active output (not open drain) */
	immr->im_cpm.cp_pbpar &= ~PB_DIGIO_MASK;	/* assignment = general i/o */
        immr->im_cpm.cp_pbdir |=  PB_DIGIO_MASK;	/* direction = out */

	/*
	 * Loop until control C pressed outputting bit pattern
	 * Change values every .5 seconds or so
	 */
	printf("\nPress <control>C to exit\n");
	while (l) {
		for (i = 0; i < PATTERNLEN; i++) {
			if (ctrlc()) {
				l = 0;
			        break;
			}
			immr->im_cpm.cp_pbdat &= ~PB_DIGIO_MASK;	/* remove existing dig i/o bits */
			immr->im_cpm.cp_pbdat |= pattern[i];		/* set new pattern on the LEDs */
			udelay (500000);				/* .5 sec sleep */
		}
	}

	immr->im_cpm.cp_pbdat &= ~PB_DIGIO_MASK;	/* clear existing dig i/o bits */
	return 0;
}

U_BOOT_CMD(
	digin, 1, 1, do_digin,
	"digin   - read & display dighital inputs\n",
	"exit with <control>C\n"
);

U_BOOT_CMD(
	digout, 1, 1, do_digout,
	"digout  - write digital outputs in sequence\n",
	"exit with <control>C\n"
);




