/*
 * (C) Copyright 2003
 * Tait Electronics Limited, Christchurch, New Zealand
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


/*
 * This file tries to bring together all the code associated with 
 * testing the DSP and its interfaces.
 * It provides a mechanism for high speed access to the HPI to allow
 * debugging as well as mechanisms that allow the external interfaces
 * of the DSP to be tested from the PowerPC
 */
 


#include <common.h>
#include <config.h>
#include <command.h>
#include <s_record.h>


extern void dsp_write_memory(int offset, ushort value);
extern ushort dsp_read_memory(int offset);
extern int cmd_get_data_size(char* arg, int default_size);


extern unsigned long codec_code_start;
extern unsigned long codec_code_length;
extern unsigned char codec_code[];

/* Some magic numbers that we use to control the code in the DSP */
#define	DSPHPI_RX_LEVEL		0x00100090
#define	DSPHPI_TX_LEVEL		0x00100092
#define	DSPHPI_FREQUENCY	0x00100095
#define	DSPHPI_FIR_ENABLE	0x00100096
#define	DSPHPI_LOOP_SET		0x001000A0
#define	DSPHPI_LOOP_READ	0x001000A2
#define	DSPHPI_WRITE_ADDR	0x00100060
#define	DSPHPI_WRITE_DATA	0x00100061
#define	DSPHPI_READ_ADDR	0x00100062
#define	DSPHPI_READ_DATA	0x00100063


/* 
 * Interpreter command to thrash the DSP HPI memory interface
 * Get a range of addresses to read from. If the last address
 * is 0x0000000 (zero) then treat it as a flag that we want to 
 * print the values out, otherwise just thrash round as fast as 
 * possible. Terminate on Control-C
 */


/* put DSP into reset by dropping and releasing reset line */
static void do_dsp_reset(void)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;

	/* apply and release reset to DSP chip to load new s/w */
	immap->im_ioport.iop_padat &= ~PA_DSP_RESET;	/* output = low */
	udelay(100);
	immap->im_ioport.iop_padat |=  PA_DSP_RESET;	/* output = high */

}

void load_dsp_binary(void)
{
	int	i;
	long	store_addr;

	store_addr = (codec_code_start / 2) + CFG_DSPHPI_START; 

	do_dsp_reset();
	udelay(100);
	for (i = 0; i <= codec_code_length; i+=2) {
	        dsp_write_memory(store_addr + i/2, (short)((codec_code[i] << 8) | codec_code[i+1]));
	}
	/* write to dsp control word to start the DSP */
	dsp_write_memory(CFG_DSPHPI_CONTROL, CFG_DSPHPI_START_CODE);

	/* wait for it to start up */
	udelay(1000);

}

int do_dsphpi ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	int l = 1, printflag = 0, i;
        ushort value[5];
	ulong addr[5];

	volatile immap_t *immr = (immap_t *) CFG_IMMR;

        /* Validate arguments */
        if ((argc < 2) || (argc > 6)){
    		printf("Usage:\n%s\n", cmdtp->usage);
        	return 1;
        }


	immr->im_ioport.iop_paodr &= ~PA_TP201;		/* active output (not open drain) */
	immr->im_ioport.iop_papar &= ~PA_TP201;		/* assignment = general i/o */
        immr->im_ioport.iop_padir |=  PA_TP201;		/* direction = out */
	immr->im_ioport.iop_padat &= ~PA_TP201;		/* set test point low to start */

	for (i = 0; i < argc - 1; i++) {
	    addr[i] = simple_strtoul(argv[i + 1], NULL, 16);
	}

	if (addr[argc-2] == 0) {
	    printflag = 1;
	}
	

	printf("\n");
	while (l) {
		if (ctrlc()) {
			l = 0;
		        break;
		}
		immr->im_ioport.iop_padat |= PA_TP201;	/* set test point high */
		value[0] = *((ushort *)addr[0]);	/* always get first address */
		immr->im_ioport.iop_padat &= ~PA_TP201;	/* set test point low */

		for (i = 1; i < argc - 2; i++) {
			value[i] = *((ushort *)addr[i]);
		}

		if (printflag) {
			printf("Values = %04X ", value[0]);
			for (i = 1; i < argc - 2; i++) {
			        printf("%04X ", value[i]);
			}
			printf("\r");
		}

	}
	printf("\n");

	return 0;
}



int do_dsp_reset_cmd ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{

	do_dsp_reset();
	return 0;
}



int do_dspaudio(int level)
{
	int 	i;
	
	/* load the code into the DSP and start it up */
	load_dsp_binary();

	/* select 1KHz audio freq */
	dsp_write_memory(DSPHPI_FREQUENCY, 1000);

	/* set level as requested */
        dsp_write_memory(DSPHPI_TX_LEVEL, level);

	/* delay to allow sampling to settle */
	for (i = 0; i < 1500; i++) {
		udelay(1000);
	}
	/* get result from DSP and display it */
	return (dsp_read_memory(DSPHPI_RX_LEVEL));

}

/* command line interface to the sine-wave generator */
int do_dspgen ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
        ushort	freq;
	ushort	result;
	ushort	fir;
	ulong	level = 0;
	int	i;

        /* Validate arguments */
        if ((argc < 2) || (argc > 5)){
    		printf("Usage:\n%s\n", cmdtp->usage);
        	return 1;
        }
	
	/* load the code into the DSP and start it up */
	load_dsp_binary();
	
	/* get the values from the user and poke then into the DSP memory */
	/* always get frequency, maybe level as well */
        freq = simple_strtoul(argv[1], NULL, 16);
	dsp_write_memory(DSPHPI_FREQUENCY, freq);

	if (argc > 2) {
	        level = simple_strtoul(argv[2], NULL, 16);
	        dsp_write_memory(DSPHPI_TX_LEVEL, (ushort)level);
	}

	/* see if FIR to be en/dis-abled */
	if (argc > 3) {
		fir = simple_strtoul(argv[3], NULL, 16);
	        dsp_write_memory(DSPHPI_FIR_ENABLE, fir);
	}

	/* see if display of input required */
	if (argc > 4) {
		puts ("Waiting for levels to stabilise\n");
		/* at least 1 second delay to start measuring input */
		for (i = 0; i < 1500; i++) {
			udelay(1000);
		}
		/* get result from DSP and display it */
		result = dsp_read_memory(DSPHPI_RX_LEVEL);
		printf ("Audio level = %04X\n", result);
	}

	/* if .b command modifier then go into interactive mode
	  to tweak the level */
	if (cmd_get_data_size(argv[0], 4) == 1) {
		puts("a-z=+/- 1%: ,-.=+/100Hz: q=quit\n");
		i = 1;
		while (i) {
		char c;
			c = serial_getc();	/* read character		*/
			switch (c & 0x7f) {
			case 'a':
				level = (level * 101) / 100;	/* up 1% */
				break;
			case 'z':
				level = (level * 99) / 100;	/* down 1% */
				break;
			case ',':
				freq -= 100;
				break;
			case '.':
				freq += 100;
				break;
			case 'q':
				i = 0;
				break;
			default:
				c = 0x07;
				serial_putc(c);
				break;
			}
			printf("Freq = %04x, Level = %04x\n", freq, level);
			dsp_write_memory(DSPHPI_FREQUENCY, freq);
		        dsp_write_memory(DSPHPI_TX_LEVEL, (ushort)level);
			
		}
	}

	return 0;
}

/* command line interface to the audio input level */
int do_dspread_cmd ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	int	i, l = 1;
	ushort	result;

	/* load the code into the DSP and start it up */
	load_dsp_binary();
	
	/* turn the output off */
	dsp_write_memory(DSPHPI_FREQUENCY, 1000);
        dsp_write_memory(DSPHPI_TX_LEVEL, 0);

	/* 1st time thru, use a 1.5 sec delay */
	for (i = 0; i < 1500; i++) {
    		udelay(1000);
	}

	while (l) {
		if (ctrlc()) {		/* abort on control-c */
			l = 0;
		        break;
		}
    		/* get result from DSP and display it */
    		result = dsp_read_memory(DSPHPI_RX_LEVEL);
    		printf ("Audio level = %04X\r", result);
		/* delay before next display */
		for (i = 0; i < 1000; i++) {
        		udelay(1000);
        	}
	}
	puts("\n");
	return 0;
}



int do_dsploop ( void )
{
        ushort result;
	
	/* load the code into the DSP and start it up */
	load_dsp_binary();

	/* enable the test by poking the enable bit */
        dsp_write_memory(DSPHPI_LOOP_SET, 1);
	
	/* wait for the test to run for a while */
	udelay(10000);
	
	/* get result from DSP */
	result = dsp_read_memory(DSPHPI_LOOP_READ);

	/* disable the test by poking the enable bit off */
        dsp_write_memory(DSPHPI_LOOP_SET, 0);

	/* turn of any squeaks and pops!! */
	do_dsp_reset();
	
	return result;
}

/* command line interface to the loopback test */
int do_dsploop_cmd ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	int result;
	
	result = do_dsploop();

	if ( result == 0xffff) {
		puts("RxEx loopback PASS\n");
	} else {
		printf ("RxEx loopback FAILED: Error = %d\n", result);
	}

	return 0;
}	

/* command line interface to the I/O test */
int do_dspio ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
        ushort data, addr;
	int	i;


	/* load the code into the DSP and start it up */
	load_dsp_binary();

        /* Validate arguments */
        if ((argc < 2) || (argc > 3)){
    		printf("Usage:\n%s\n", cmdtp->usage);
        	return 1;
        }
	
	/* get the values from the user and poke then into the DSP memory */
	/* always get address, maybe data (for writing) as well */
        addr = simple_strtoul(argv[1], NULL, 16);

	if (argc > 2) {		/* use write command */
	        data = simple_strtoul(argv[2], NULL, 16);
	        dsp_write_memory(DSPHPI_WRITE_DATA, data);	/* data first, then address */
	        dsp_write_memory(DSPHPI_WRITE_ADDR, addr);
		
	} else {
		dsp_write_memory(DSPHPI_READ_ADDR, addr);
		for (i = 0; i < 100; i++) {
			if (dsp_read_memory(DSPHPI_READ_ADDR) == 0xffff)
				continue;		
			udelay(100);
		}
		data = dsp_read_memory(DSPHPI_READ_DATA);
		printf("DSP offset %04x=%04x\n", addr, data);
	}

	return 0;
}	


//#if 0	/* S-record load no longer used */

static int load_dsp(ulong source_ptr);
static int read_record (char *buf, ulong len);

static char * srecord_ptr = NULL;	/* pointer to s-record DSP code */

/* ------------------------------------------------------------------------- */

int do_dspload ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
        ulong	address;

	/* get the load address from the user */
        address = simple_strtoul(argv[1], NULL, 16);

	/* load the code into the DSP and start it up */
	if (load_dsp(address)) {
		puts ("DSP download failure\n");
		return 1;
	}

	return 0;
}	




/* ------------------------------------------------------------------------- */
/* function to read an ASCII file held in memory in S-record format and
 * download it to the DSP
 */

static int load_dsp(ulong source_ptr)
{
	char	record[SREC_MAXRECLEN + 1];	/* buffer for one S-Record	*/
	char	binbuf[SREC_MAXBINLEN];		/* buffer for binary data	*/
	int	binlen;				/* no. of data bytes in S-Rec.	*/
	int	type;				/* return code for record type	*/
	ulong	addr;				/* load address from S-Record	*/
	ulong	size;				/* number of bytes transferred	*/
	int	i;
	ulong	store_addr;
	ulong	start_addr = ~0;
	ulong	end_addr   =  0;

	do_dsp_reset();
	
	srecord_ptr = (char *)source_ptr;
	while (read_record(record, SREC_MAXRECLEN + 1) >= 0) {
		type = srec_decode (record, &binlen, &addr, binbuf);

		if (type < 0) {
			return (~0);		/* Invalid S-Record		*/
		}

		switch (type) {
		case SREC_DATA2:
		case SREC_DATA3:
		case SREC_DATA4:
		    store_addr = (addr / 2) + CFG_DSPHPI_START; 
		    
		    for (i = 0; i <= binlen; i+=2) {
			    dsp_write_memory(store_addr + i/2, (short)((binbuf[i] << 8) | binbuf[i+1]));
		    }
		    
		    if ((store_addr) < start_addr)
			start_addr = store_addr;
		    if ((store_addr + binlen - 1) > end_addr)
			end_addr = store_addr + binlen - 1;
		    break;
		case SREC_END2:
		case SREC_END3:
		case SREC_END4:
		    udelay (10000);
		    size = end_addr - start_addr + 1;
		    debug ("\n"
			    "## First Load Addr = 0x%08lX\n"
			    "## Last  Load Addr = 0x%08lX\n"
			    "## Total Size      = 0x%08lX = %ld Bytes\n",
			    start_addr, end_addr, size, size
		    );
		    flush_cache (addr, size);
		/* write to dsp control word to start the DSP */
		    dsp_write_memory(CFG_DSPHPI_CONTROL, CFG_DSPHPI_START_CODE);
		    return 0;
		case SREC_START:
		    break;
		default:
		    break;
		}
	}

	return (~0);			/* Download aborted		*/
}

static int
read_record (char *buf, ulong len)
{
	char *p;
	char c;

	--len;	/* always leave room for terminating '\0' byte */

	for (p=buf; p < buf+len && srecord_ptr != NULL; ++p) {
		c = *srecord_ptr++;		/* read character		*/
		switch (c) {
		case '\r':
		case '\n':
			*p = '\0';
			return (p - buf);
		case '\0':
		case 0x03:			/* ^C - Control C		*/
			return (-1);
		default:
			*p = c;
		}

	    /* Check for the console hangup */
		if (ctrlc()) {
		    return (-1);
		}
	}

	/* line too long - truncate */
	*p = '\0';
	return (p - buf);
}

/* ------------------------------------------------------------------------- */

U_BOOT_CMD(
	dspload, 2, 0, do_dspload,
	"dspload - load the DSP with code from a defined address\n",
	"address\n"
);
//#endif	/* S-record load no longer used */

U_BOOT_CMD(
	thrash,	6, 0, do_dsphpi,
	"thrash  - thrash DSP HPI memory interface\n",
	"address1 [[address2..n] [printflag]]\n"
);

U_BOOT_CMD(
	dspreset, 1, 1,	do_dsp_reset_cmd,
	"dspreset- waggle DSP reset line\n",
	"put DSP into a known state\n"
);

U_BOOT_CMD(
	dspgen, 5, 1, do_dspgen,
	"dspgen  - generate sinewave from DSP\n",
	"[.b] frequency (Hz) [level (0-7fff)] [FIR] [read flag]\n"
);

U_BOOT_CMD(
	dspread, 1, 1, do_dspread_cmd,
	"dspread - display analogue input level\n",
	"displays input level in hex\n"
);

U_BOOT_CMD(
	dsploop, 1, 1, do_dsploop_cmd,
	"dsploop - loop test DSP McBSP1 interface\n",
	"returns PASS or error 0-7\n"
);

U_BOOT_CMD(
	dspio, 3, 1, do_dspio,
	"dspio   - read/write DSP I/O interface\n",
	"offset [data]\n"
);


