/*
 * (C) Copyright 2003
 * Tait Electronics Limited, Christchurch, New Zealand
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


/*
 * This file provides a simple command interface to all the loopback
 * tests that currently exist in the POST and DSP sections
 */
 


#include <common.h>
#include <config.h>
#include <command.h>


extern int fec_post_test (int flags);
extern int uart_test_ctlr (int ctlr, int index);
extern int do_dspaudio(int level);
extern int do_dsploop ( void );
extern short get_digio_value(void);


#ifdef CONFIG_POST
/* these commands are only available if the POST code is enabled */
int do_loop_smc(void)
{

	puts("Testing SMC - ");
	return (uart_test_ctlr (0, 1));
}

int do_loop_scc(void)
{

	puts("Testing SCC - ");
	return (uart_test_ctlr (1, 0));
}

int do_loop_fec(void)
{

	puts("Testing FEC - ");
	return (fec_post_test (0));

}
#else
int do_loop_smc(void)
{

	puts("Testing SMC - skipping - ");
}

int do_loop_scc(void)
{

	puts("Testing SCC - skipping - ");
	return 0;
}

int do_loop_fec(void)
{

	puts("Testing FEC - skipping - ");
	return 0;

}
#endif

int do_loop_dspio(void)
{
	int ret;
	
	
	puts("Testing DSPIO - ");

	ret = do_dsploop();

	if (ret == 0xffff) {
		ret = 0;
	} else {
		printf("Failed at state = %d\n", ret);
		ret = 1;
	}

	return ret;

}




/*
 * Define the digital input and output lines more exactly!!
 *
 * Outputs
 * PB14	0x00020000	M (on analog connector)
 * PB15	0x00010000	DIG_IN4/ANT_RLY (bi-directional)
 * PB16	0x00008000	HS_DIG_IN_0
 * PB17	0x00004000	HS_DIG_IN_1
 * PB18	0x00002000	HS_DIG_IN_2
 * PB19	0x00001000	TX_KEY
 * PB22 0x00000200	DIG6 (RSSI+)
 * PB23 0x00000100      DIG7 (RSSI-)
 * PB25 0x00000040	TXD
 
 * Inputs
 * PA3/IRQ1	0x3000	1PPS
 * PC12/IRQ6	0x4800	E (on analog connector)
 * PC13		0x0400	PWR_EX
 * PC14		0x0200	PWR_RX
 * PC15		0x0100	PWR_ON
 * (on 9 pin connector)
 * IPA0		0x0080	DIG_IN0
 * IPA1		0x0040	DIG_IN1
 * IPA2		0x0020	DIG_IN2
 * IPA3		0x0010	DIG_IN3
 * IPA4		0x0008	DIG_IN4/ANT_RLY (bi-directional)
 * IPA5		0x0004	DIG_IN5 (also PB24 SMC0 RX data!!)
 * IPA6		0x0002	RX_GATE
 * IPA7		0x0001	COAX_RLY_DRV
 */

struct digio_test {
    ulong	output;
    ushort	input_mask;
    ushort	expected;	 	
    char *	msg;
} digio_list[] = 

{
    {
	0x00020000,
	0x4800,
	0x4800,
	"Analogue E/M"
    },
    {
	0x00001000,
	0x3000,
	0x0000,
	"TX Key/1pps"
    },
    {
	0x00002000,
	0x0400,
	0x0400,
	"HS_DIG_IN2/PWR_EX"
    },
    {
	0x00000040,
	0x00c0,
	0x00c0,
	"TXD/DIG_IN0+DIG_IN1"
    },
    {
	0x00004000,
	0x0300,
	0x0300,
	"HS_DIG_IN1/PWR_RX+PWR_ON"
    },
    {
	0x00008000,
	0x0003,
	0x0003,
	"HS_DIG_IN0/RX_GATE+COAX_RLY_DRV"
    },
    {
	0x00010000,
	0x0030,
	0x0030,
	"ANT_RLY/DIG_IN2+DIG_IN3"
    },
    {
	0x00000100,
	0x0008,
	0x0008,
	"RSSI/DIG_IN4"
    }
};

#define DIGIO_LIST_LEN	(sizeof(digio_list) / sizeof(digio_list[0]))


int do_loop_digio(void)
{

	int i;
	ushort value1, value2;
	ulong saveodr, savepar, savedir;
	int ret = 0;
	volatile immap_t *immr = (immap_t *) CFG_IMMR;


	saveodr = immr->im_cpm.cp_pbodr;
	savepar = immr->im_cpm.cp_pbpar;
	savedir = immr->im_cpm.cp_pbdir;

	puts("Testing DIGIO - ");
	/* Use an array to define the output signal sequence and check for
	 * the correct returned value for each value output
	 * Provide a likely fault summary in the case of failure 
	 * All outputs are on port B, inputs we get from whereever!! */


	/* work our way down the list */
	for (i = 0; i < DIGIO_LIST_LEN; i++) {
	
	/* Output true value - put port bit into correct mode for each bit */
		immr->im_cpm.cp_pbodr &= ~digio_list[i].output;	/* active output (not open drain) */
		immr->im_cpm.cp_pbpar &= ~digio_list[i].output;	/* assignment = general i/o */
	        immr->im_cpm.cp_pbdir |=  digio_list[i].output;	/* direction = out */
		immr->im_cpm.cp_pbdat &= ~digio_list[i].output;	/* remove existing dig i/o bits */
		immr->im_cpm.cp_pbdat |=  digio_list[i].output;	/* output true value */
		udelay(100000);
		
	/* look for input & mask it */
		value1 = get_digio_value() & digio_list[i].input_mask;
		
	/* Output false value */
		immr->im_cpm.cp_pbdat &= ~digio_list[i].output;
		udelay(100000);
		
	/* look for input & mask it */
		value2 = get_digio_value() & digio_list[i].input_mask;

	/* put pin assignment back to dedicated peripheral use if not a general I/O pin */

		immr->im_cpm.cp_pbdir = 0;		/* direction = in */
		immr->im_cpm.cp_pbpar = ~PB_DIGIO_MASK;	/* assignment = peripheral on all except I/O */
		immr->im_cpm.cp_pbdir = 0;		/* select peripheral 0 */

	/* Check input */
		if ((value1 != digio_list[i].expected) 
		    || (value2 != (~digio_list[i].expected & digio_list[i].input_mask))) {
	/* Display helpful error message if a problem */
			if (!ret)
			        puts("\n");			/* new line 1st time */
		        printf ("Failed line = %s\n", digio_list[i].msg);
			debug ("\t\t\tval1 %04x, exp %04x, val2 %04x, exp %04x\n",value1, digio_list[i].expected,
			    value2, (~digio_list[i].expected & digio_list[i].input_mask));
		        ret = 1;
			udelay(1000);		/* last char should get out but sometimes it don't */
		}	    
	/* loop until all lines in list done */
	}
	
	/* restore port B to what it was */
	immr->im_cpm.cp_pbodr = saveodr;
	immr->im_cpm.cp_pbpar = savepar;
	immr->im_cpm.cp_pbdir = savedir;

	return ret;

}


int do_loop_audio(void)
{
        int ret;
	
	
	puts("Testing AUDIO - ");
	ret = do_dspaudio(8000);
	/* expect about 1.5 times what we put in 
	 * so look for x1 to x2 */
	if ((ret < 8000) || (ret > 16000)) {
		ret = 1;
	} else {
		ret = 0;
	}
	
	return ret;

}


void pass_fail(int ret)
{
	if (ret) {
		puts ("FAILED\n");
	} else {
		puts ("PASSED\n");
	}			
}



int do_loop ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	int	i, l;
    
    
	if (argc < 2) {
    		printf("Usage:\n%s\n", cmdtp->usage);
        	return 1;
        }
		
	if (strcmp (argv[1], "all") == 0) {
		/* run all tests sequentially */
		pass_fail(do_loop_smc());
		pass_fail(do_loop_scc());
		pass_fail(do_loop_fec());
		pass_fail(do_loop_dspio());
		pass_fail(do_loop_digio());
		pass_fail(do_loop_audio());
	
	} else {
		/* Run individual tests */
		for (i = 1; i < argc; i++) {
			l = strlen(argv[i]);
			if        (strncmp (argv[i], "smc", l) == 0) {
				pass_fail(do_loop_smc());
			} else if (strncmp (argv[i], "scc", l) == 0) {
				pass_fail(do_loop_scc());
			} else if (strncmp (argv[i], "fec", l) == 0) {
				pass_fail(do_loop_fec());
			} else if (strncmp (argv[i], "dspio", l) == 0) {
				pass_fail(do_loop_dspio());
			} else if (strncmp (argv[i], "digio", l) == 0) {
				pass_fail(do_loop_digio());
			} else if (strncmp (argv[i], "audio", l) == 0) {
				pass_fail(do_loop_audio());
			} else {
				printf("Unknown subcommand %s\n", argv[i]);
				return 1;
			}
		}
	}

	return 0;

}



/* ------------------------------------------------------------------------- */

U_BOOT_CMD(
	looptst,	6,	0,	do_loop,
	"looptst - loopback test SMC, SCC, FEC, DSPIO, AUDIO, DIGIO\n",
	"[[ALL]|SMC|SCC...]]\n"
);


