/*
*
* Do not edit $tags$ manually, CVS will take care of this!
*
* Copyright (c) 2003 Tait Electronics Ltd, All rights reserved.
*
* Source File Name:   $RCSfile: hpitest.c,v $
*
* Module Name:        Test HPI memory with random walk reads
*
* Sub-System Name:    U-Boot external program
*
* Description:
* A quicky program that is being used to debug the HPI and check that it doesn't fail under
* stress conditions
* RISC repository
* Revision:            $Revision: 1.1 $
*
* Date:                $Date: 2004/12/08 03:03:42 $
*
* Author:              Rob Searle, Seung Lee, Dmytro Bablinyuk, Robert Ancell, Robin Gilks
*
* Locked By:           $Locker:  $
*
* This file has never been reviewed
*
*/

#include <asm/byteorder.h>

#include <common.h>
#include <exports.h>
#include <logbuff.h>



/*
* NOTE
* Order of code is important here - the bootloader external programs
* expect the entry point to be at the start of the program and to
* have the correct name.
*/

static unsigned long my_strtoul(const char *cp);
static unsigned char rand(void);

int hpitest(int argc, char *argv[]){

    long address = 0;
    int i, l = 1;
    short array[256], v, *p;
      
    app_startup(argv);

    if (argc != 2){
	printf("Usage: hpitest <address>\n", argv[0]);
        return 1;
    }

  
    address = my_strtoul(argv[1]);
    
/* 
 * Fill array from hpi memory starting at <address> using very 
 * conservative method (multiple reads). 
 * Then random walk across the memory block doing fast reads and see if
 * they are the same. Report errors as they are found
 */

    for (i = 0; i < 256; i++)
    {
	p = (short *)address + i;
	array[i] = *p;
	udelay(5);
	array[i] = *p;
	udelay(5);
	array[i] = *p;
    }
    
    while (l) {
	if (ctrlc()) {
		l = 0;
	        break;
	}
	i = rand();
	p = (short *)address + i;
	v = *p;
	if (v != array[i])
	{
	    printf ("Error at offset %02x, was %04x, should be %04x\n", i, v, array[i]);
	}
    }

    return 0;

}


/* very simple ctype for isdigit etc */

#define _U	0x01	/* upper */
#define _L	0x02	/* lower */
#define _D	0x04	/* digit */
#define _X	0x08	/* hex digit */


#define __ismask(x) (_ctype[(int)(unsigned char)(x)])

#define isdigit(c)	((__ismask(c)&(_D)) != 0)
#define islower(c)	((__ismask(c)&(_L)) != 0)
#define isxdigit(c)	((__ismask(c)&(_D|_X)) != 0)


unsigned char _ctype[] = {
0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,
_D,_D,_D,_D,_D,_D,_D,_D,			/* 48-55 */
_D,_D,0,0,0,0,0,0,
0,_U|_X,_U|_X,_U|_X,_U|_X,_U|_X,_U|_X,_U,	/* 64-71 */
0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,
0,_L|_X,_L|_X,_L|_X,_L|_X,_L|_X,_L|_X,_L,	/* 96-103 */
0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0};


static inline unsigned char toupper(unsigned char c)
{
	if (islower(c))
		c -= 'a'-'A';
	return c;
}




static unsigned long my_strtoul(const char *cp)
{
    unsigned long result = 0,value;
    int base = 16;

    while (isxdigit(*cp) && (value = isdigit(*cp) ? *cp-'0' : (islower(*cp)
	    ? toupper(*cp) : *cp)-'A'+10) < base) {
	result = result*base + value;
	cp++;
    }
    return result;
}

static unsigned char rand(void)
{
    static short random = 0x2912;
    short t1, t2;
    
    t1 = ((random & 0x8000) >> 15) ^ ((random & 0x1000) >> 12);
    random = (random << 1) | t1;
    
    t2 = (random & 0xff) ^ ((random >> 8) & 0xff);
    
    return t2;
}

int ctrlc (void)
{
    if (tstc ()) {
	switch (getc ()) {
	case 0x03:		/* ^C - Control C */
	    return 1;
	default:
	    break;
	}
    }
    return 0;
}

